@@ -2,7 +2,7 @@
 #+
 # file: AuthRegister.pm
 # CGI::AuthRegister - AuthRegister Module for Simple CGI Authentication and Registration in Perl
-# (c) 2012-18 Vlado Keselj http://web.cs.dal.ca/~vlado
+# (c) 2012-19 Vlado Keselj http://web.cs.dal.ca/~vlado
 # $Date: $
 # $Id: $
 #-
@@ -14,7 +14,7 @@ use strict;
 use vars qw($NAME $ABSTRACT $VERSION);
 $NAME     = 'AuthRegister';
 $ABSTRACT = 'AuthRegister Module for Simple CGI Authentication and Registration in Perl';
-$VERSION  = '1.1';
+$VERSION  = '1.2';
 #-
 use CGI qw(:standard);
 # Useful diagnostics:
@@ -39,7 +39,7 @@ use vars qw(@ISA @EXPORT);
 use vars qw( $AddAuthenticatedUser
   $DBdir $DBusers $DBpwd $DBsessions $DBusersCas $DBpwdCas
   $DBsessionsCas $DBcasTokens $DebugLevel
-  $Email_from $Email_bcc $Error $ErrorInternal
+  $Email_admin $Email_from $Email_bcc $Error $ErrorInternal
   $GenCasPageCustom $LogReport
   $LDAPuse $LDAPserver $LDAPdn $LDAPaddUsers $LinkForgotpwd
   $Sendmail $Session $SessionId $SiteId $SiteName $Ticket
@@ -174,16 +174,23 @@ sub run_cas {
       $page=~ s/<!--!hiddenfields-->/$h\n$&/;
       $h = "<input type=\"hidden\" name=\"stoken\" value=\"$stoken\">";
       $page=~ s/<!--!hiddenfields-->/$h\n$&/;
-      $page =~ s/(<input class="inputButton" value=)"Login"/$1"Proceed_Back"/;
+      $page =~ s/(<input class="inputButton" value=)"Login"/$1"Proceed"/;
       my $r = &encodeuri($redirect_uri);
       $page =~ s/(<form id="login_form" action=)"\?login"/$1"$r"/;
     } else { $page =~ s/<input class="inputButton".*?>//s; }
-    print $page; exit;
+    print $page;
+    # Log out user so that they have to login every time they use the service
+    logout();
+    exit;
   };
   ### End of helper functions
   
   # Check redirect_uri
-  if ($redirect_uri ne '' && $redirect_uri !~ /^https:\/\/(\w|[-.~\/])+/i) {
+  if ($redirect_uri ne '' &&
+      $redirect_uri !~ /^https:\/\/(\w|[-.~\/])+/i &&
+      $redirect_uri !~ /^http:\/\/(\w|[-.~\/:])+/i  ## This is temporary for a student project
+                                                   ## It is probably is too relaxed.
+     ) {
     my $page = &gen_cas_page;
     my $h = 'redirect_uri Error!';
     my $t = "URI of the requesting site is not in an acceptable format:<br>\n".
@@ -343,6 +350,8 @@ sub require_login {
 # parameters:
 #   -return_status=>1  rather than exiting on failure, return status
 #         return status values: 'logged out', 1, 'not logged in' 'login failed'
+#         If we want that user gets a suggestion to use CAS to login, then
+#         this option should not be used.
 #
 sub _require_login_using_cas {
   my %args = @_; my $casurl = $args{-cas};
@@ -363,7 +372,7 @@ sub _require_login_using_cas {
   if ($SessionId ne '') { print header(); return 1; }
 
   my $request_type = param('request_type');
-  if ($request_type ne 'Proceed_Back') {
+  if ($request_type ne 'Proceed') {
     print header(); if ($retStatus) { return 'not logged in' }
     print $HTMLstart, $LoginMsg; exit; }
   my $username = param('username'); my $stoken = param('stoken');
@@ -389,14 +398,14 @@ sub _require_login_using_cas {
       "HTTP POST error message: ".$resp->message."\n";
   }
   if ($result ne 'ok') {
-    $Error.="ERR-384:verify failed, result=($result) casurl=($casurl)\n";
+    $Error.="ERR-401:verify failed, result=($result) casurl=($casurl)\n";
     print header(); $LogReport.=$Error; &store_log;
     if ($retStatus) { return 'login failed'; }
     print $HTMLstart, "Unsuccessful login!\n"; exit; }
   my $u = ($AddAuthenticatedUser ? &get_user_by_userid_or_add($username) :
 	   &get_user_unique('userid', $username));
   if ($u eq '') {
-    $Error.="382-ERR: no userid ($username)\n";
+    $Error.="408-ERR: no userid ($username)\n";
     $LogReport.=$Error; &store_log;
     print header(); if ($retStatus) { return 'login failed'; }
     print $HTMLstart, "Unsuccessful login!\n"; &store_log; exit; }
@@ -562,9 +571,9 @@ sub login {
 sub _login_ldap_add {
   my $userid = shift; my $password = shift;
   if (!&password_check_ldap($userid, $password)) {
-    $Error.="539-ERR:Invalid password for LDAP\n"; return ''; }
+    $Error.="570-ERR:Invalid password for LDAP\n"; return ''; }
   my $u = &get_user_by_userid_or_add($userid);
-  if ($u eq '') { $Error.="541-ERR:\n"; &store_log; return; }
+  if ($u eq '') { $Error.="572-ERR:\n"; &store_log; return; }
   $User = $u;
   # Randomize more salt
   $SecretSalt = md5_base64("$SecretSalt $password");
@@ -578,7 +587,7 @@ sub set_new_session {
   my $email = $u->{email};
   my $userid = $u->{userid};
   if ($email !~ /@/ && $userid !~ /\w/) {
-    $Error .= "555-ERR: No email nor userid\n"; return '';  }
+    $Error .= "586-ERR: No email nor userid\n"; return '';  }
   my $sDir = "$DBdir/$DBsessions";
   if (!-d $sDir && !&check_db_files) { return ''; }
 
@@ -595,8 +604,11 @@ sub set_new_session {
   my $sessionrecord = "SessionId:$SessionId\nTicket:$Ticket\n";
   $sessionrecord.="email:$email\n" if $email ne '';
   $sessionrecord.="userid:$userid\n" if $userid ne '';
-  putfile("$sDir/$SessionId/session.info", $sessionrecord);
+  my $sessioninfofile = "$sDir/$SessionId/session.info";
+  putfile($sessioninfofile, $sessionrecord);
   $UserEmail = $email; $UserId = $userid; $User = $u;
+  $Session = &read_db_record("file=$sessioninfofile");
+  die unless ref($Session);
   return $SessionId;
 }
 
@@ -1645,14 +1657,27 @@ designed with the assumption that the CGI programs run with user uid.
 
 =head1 PREDEFINED VARIABLES
 
+=head2 $CGI::AuthRegister::Email_admin
+
 =head2 $CGI::AuthRegister::Email_bcc
 
 For example,
 
-  $CGI::AuthRegister::Email_bcc = 'Vlado Keselj <vlado+ar@cs.dal.ca>';
+  $CGI::AuthRegister::Email_bcc = 'Vlado Keselj <vlado+bcc@dnlp.cad>';
 
 If nonempty, causes BCC copies of the emails to be sent to this address.
-This is typically an administrator's address.
+This is typically an administrator's address.  If Email_admin is empty and
+Email_bcc is not, then Email_bcc is used as Email_admin.
+
+=head2 $CGI::AuthRegister::Email_from
+
+Example:
+  $CGI::AuthRegister::Email_from = $CGI::AuthRegister::SiteId
+      . ' <vlado@dnlp.ca>;
+
+=head2 $CGI::AuthRegister::SendLogs
+
+If true, the log entries will be sent by email to the admin.
 
 =head1 FUNCTIONS
 
